@@ -32,6 +32,13 @@ struct VisitorDeclaration : hilti::visitor::PreOrder<cxx::declaration::Type, Vis
     auto typeID(const Node& n) { return n.as<Type>().typeID(); }
     auto cxxID(const Node& n) { return n.as<Type>().cxxID(); }
 
+    result_t operator()(const type::Bitfield& n) {
+        for ( const auto& b : n.bits(true) )
+            addDependency(b.itemType());
+
+        return {};
+    }
+
     result_t operator()(const type::Struct& n, const position_t p) {
         assert(typeID(p.node));
 
@@ -53,6 +60,8 @@ struct VisitorDeclaration : hilti::visitor::PreOrder<cxx::declaration::Type, Vis
                 std::vector<cxx::declaration::Argument> args;
                 std::vector<cxx::type::struct_::Member> fields;
 
+                cxx::Block ctor;
+
                 cxx::Block self_body;
                 self_body.addStatement(util::fmt("return ::hilti::rt::ValueReference<%s>::self(this)", id));
 
@@ -199,6 +208,12 @@ struct VisitorDeclaration : hilti::visitor::PreOrder<cxx::declaration::Type, Vis
                         t = fmt("std::optional<%s>", t);
 
                     std::optional<cxx::Expression> default_;
+
+                    // Push a block so all values needed to default-initialize
+                    // fields have a block. This is required if compiling the
+                    // value needs to e.g., create temporaries.
+                    cg->pushCxxBlock(&ctor);
+
                     if ( ! f.isOptional() ) {
                         cg->pushSelf("__self()");
                         if ( auto x = f.default_() )
@@ -208,11 +223,16 @@ struct VisitorDeclaration : hilti::visitor::PreOrder<cxx::declaration::Type, Vis
                         cg->popSelf();
                     }
 
-                    auto x = cxx::declaration::Local{cxx::ID(f.id()),
-                                                     t,
-                                                     {},
-                                                     default_,
-                                                     (f.isStatic() ? "inline static" : "")};
+                    cg->popCxxBlock();
+
+                    if ( default_ )
+                        ctor.addStatement(fmt("%s = %s", cxx::ID(f.id()), *default_));
+
+                    // Do not pass a default value here since initialization of
+                    // the member happens (and needs to happen) via the ctor
+                    // block above to guarantee we have a block.
+                    auto x = cxx::declaration::Local{cxx::ID(f.id()), t, {}, {}, (f.isStatic() ? "inline static" : "")};
+
 
                     fields.emplace_back(std::move(x));
                 }
@@ -245,6 +265,7 @@ struct VisitorDeclaration : hilti::visitor::PreOrder<cxx::declaration::Type, Vis
                 auto t = cxx::type::Struct{.args = std::move(args),
                                            .members = std::move(fields),
                                            .type_name = cxx::ID(id.local()),
+                                           .ctor = std::move(ctor),
                                            .add_ctors = true};
                 return cxx::declaration::Type{id, t, t.inlineCode()};
             });
@@ -365,6 +386,14 @@ struct VisitorStorage : hilti::visitor::PreOrder<CxxTypes, VisitorStorage> {
 
     result_t operator()(const type::Bool& n) { return CxxTypes{.base_type = "::hilti::rt::Bool"}; }
 
+    result_t operator()(const type::Bitfield& n) {
+        auto x = node::transform(n.bits(true), [this](const auto& b) {
+            return cg->compile(b.itemType(), codegen::TypeUsage::Storage);
+        });
+        auto t = fmt("hilti::rt::Bitfield<%s>", util::join(x, ", "));
+        return CxxTypes{.base_type = t};
+    }
+
     result_t operator()(const type::Bytes& n) { return CxxTypes{.base_type = "::hilti::rt::Bytes"}; }
 
     result_t operator()(const type::Real& n) { return CxxTypes{.base_type = "double"}; }
@@ -792,6 +821,19 @@ struct VisitorTypeInfoDynamic : hilti::visitor::PreOrder<cxx::Expression, Visito
     auto typeID(const Node& n) { return n.as<Type>().typeID(); }
     auto cxxID(const Node& n) { return n.as<Type>().cxxID(); }
 
+    result_t operator()(const type::Bitfield& n, position_t p) {
+        std::vector<std::string> elems;
+        auto ttype = cg->compile(p.node.as<Type>(), codegen::TypeUsage::Storage);
+
+        for ( const auto&& [i, b] : util::enumerate(n.bits()) )
+            elems.push_back(fmt(
+                "::hilti::rt::type_info::bitfield::Bits{ \"%s\", %s, hilti::rt::bitfield::elementOffset<%s, %d>() }",
+                b.id(), cg->typeInfo(b.itemType()), ttype, i));
+
+        return fmt("::hilti::rt::type_info::Bitfield(std::vector<::hilti::rt::type_info::bitfield::Bits>({%s}))",
+                   util::join(elems, ", "));
+    }
+
     result_t operator()(const type::Enum& n) {
         std::vector<std::string> labels;
 
@@ -867,9 +909,9 @@ struct VisitorTypeInfoDynamic : hilti::visitor::PreOrder<cxx::Expression, Visito
             if ( auto x = cxxID(p.node) )
                 cxx_type_id = *x;
 
-            fields.push_back(fmt("::hilti::rt::type_info::struct_::Field{ \"%s\", %s, offsetof(%s, %s), %s%s }",
+            fields.push_back(fmt("::hilti::rt::type_info::struct_::Field{ \"%s\", %s, offsetof(%s, %s), %s, %s%s }",
                                  cxx::ID(f.id()), cg->typeInfo(f.type()), cxx_type_id, cxx::ID(f.id()), f.isInternal(),
-                                 accessor));
+                                 f.isAnonymous(), accessor));
         }
 
         return fmt("::hilti::rt::type_info::Struct(std::vector<::hilti::rt::type_info::struct_::Field>({%s}))",
@@ -1065,7 +1107,7 @@ const CxxTypeInfo& CodeGen::_getOrCreateTypeInfo(const hilti::Type& t) {
         // Prefer the bare type name as the display value.
         display << *t.typeID();
     else
-        hilti::print(display, t);
+        hilti::print(display, t, true);
 
     if ( display.str().empty() )
         logger().internalError(fmt("codegen: type %s does not have a display rendering for type information",
@@ -1102,7 +1144,7 @@ const CxxTypeInfo& CodeGen::_getOrCreateTypeInfo(const hilti::Type& t) {
                 logger().internalError(fmt("codegen: type %s does not have a dynamic type info visitor", t), t);
 
             auto id_init = (t.typeID() ? fmt("\"%s\"", *t.typeID()) : std::string("{}"));
-            auto init = fmt("{ %s, \"%s\", new %s }", id_init, display.str(), *x);
+            auto init = fmt("{ %s, \"%s\", new %s }", id_init, util::escapeBytesForCxx(display.str()), *x);
 
             ti.declaration =
                 cxx::declaration::Constant{.id = tid, .type = "::hilti::rt::TypeInfo", .init = init, .linkage = ""};
