@@ -143,6 +143,7 @@ std::string cxx::normalize_id(std::string id) {
             id.append("_");
 
         id = util::replace(id, "%", "0x25");
+        id = util::replace(id, "@", "0x40");
         id = util::replace(id, "~", "_0x7e_"); // we expect to see this only at the beginning (for "~finally")
         return id;
     };
@@ -341,7 +342,9 @@ std::string cxx::type::Struct::str() const {
 
     auto fmt_member = [&](const auto& f) {
         if ( auto x = std::get_if<declaration::Local>(&f) ) {
-            if ( ! (x->isInternal() || x->linkage == "inline static") ) // Don't visit internal or static fields.
+            if ( x->isAnonymous() )
+                visitor_calls.emplace_back(fmt("_(\"<anon>\", %s); ", x->id));
+            else if ( ! (x->isInternal() || x->linkage == "inline static") ) // Don't visit internal or static fields.
                 visitor_calls.emplace_back(fmt("_(\"%s\", %s); ", x->id, x->id));
 
             // We default initialize any members here that don't have an
@@ -454,12 +457,18 @@ std::string cxx::type::Struct::inlineCode() const {
     });
 
     auto init_locals_user = [&]() {
-        return util::join(util::transform(locals_user,
-                                          [&](const auto& x) {
-                                              auto& l = std::get<declaration::Local>(x);
-                                              return l.init ? fmt("    %s = %s;\n", l.id, *l.init) : std::string();
-                                          }),
-                          "");
+        cxx::Formatter init;
+        init.compact_block = false;
+        init.ensure_braces_for_block = false;
+        init << ctor;
+
+        return init.str() + util::join(util::transform(locals_user,
+                                                       [&](const auto& x) {
+                                                           auto& l = std::get<declaration::Local>(x);
+                                                           return l.init ? fmt("    %s = %s;\n", l.id, *l.init) :
+                                                                           std::string();
+                                                       }),
+                                       "");
     };
 
     auto init_locals_non_user = [&]() {
