@@ -39,7 +39,7 @@ constexpr int OPT_CXX_LINK = 1000;
 constexpr int OPT_CXX_ENABLE_DYNAMIC_GLOBALS = 1001;
 
 static struct option long_driver_options[] = {{"abort-on-exceptions", required_argument, nullptr, 'A'},
-                                              {"show-backtraces", required_argument, nullptr, 'B'},
+                                              {"show-backtraces", no_argument, nullptr, 'B'},
                                               {"compiler-debug", required_argument, nullptr, 'D'},
                                               {"cxx-enable-dynamic-globals", no_argument, nullptr,
                                                OPT_CXX_ENABLE_DYNAMIC_GLOBALS},
@@ -1167,14 +1167,20 @@ Result<Nothing> Driver::jitUnits() {
 void Driver::printHiltiException(const hilti::rt::Exception& e) {
     std::cerr << fmt("uncaught exception %s: %s", util::demangle(typeid(e).name()), e.what()) << std::endl;
 
-    if ( _driver_options.show_backtraces ) {
-        if ( auto bt = e.backtrace(); ! bt->empty() ) {
-            std::cerr << "backtrace:\n";
+    if ( ! _driver_options.show_backtraces )
+        return;
 
-            for ( const auto& s : *bt )
-                std::cerr << "  " << s << "\n";
-        }
-    }
+    if ( ! e.backtrace() )
+        return;
+
+    auto bt = e.backtrace()->backtrace();
+    if ( bt->empty() )
+        return;
+
+    std::cerr << "backtrace:\n";
+
+    for ( const auto& s : *bt )
+        std::cerr << "  " << s << "\n";
 }
 
 Result<Nothing> Driver::initRuntime() {
