@@ -2,11 +2,12 @@
 
 #include "spicy/ast/types.h"
 
+#include <hilti/ast/types/bitfield.h>
 #include <hilti/ast/types/bytes.h>
 #include <hilti/ast/types/integer.h>
 #include <hilti/ast/types/regexp.h>
 
 bool spicy::type::supportsLiterals(const hilti::Type& t) {
     return t.isA<hilti::type::Bytes>() || t.isA<hilti::type::RegExp>() || t.isA<hilti::type::SignedInteger>() ||
-           t.isA<hilti::type::UnsignedInteger>();
+           t.isA<hilti::type::UnsignedInteger>() || t.isA<hilti::type::Bitfield>();
 }
