@@ -206,7 +206,7 @@ class DNS_Request
 public:
 	DNS_Request(std::string host, int request_type, bool async = false);
 	DNS_Request(const IPAddr& addr, bool async = false);
-	~DNS_Request();
+	~DNS_Request() = default;
 
 	std::string Host() const { return host; }
 	const IPAddr& Addr() const { return addr; }
@@ -241,8 +241,6 @@ DNS_Request::DNS_Request(const IPAddr& addr, bool async) : addr(addr), async(asy
 	request_type = T_PTR;
 	}
 
-DNS_Request::~DNS_Request() { }
-
 void DNS_Request::MakeRequest(ares_channel channel, DNS_Mgr* mgr)
 	{
 	// This needs to get deleted at the end of the callback method.
@@ -1598,7 +1596,7 @@ TableValPtr DNS_Mgr::empty_addr_set()
 
 // Unit testing coverage for the DNS_Mgr code, including making actual DNS requests to
 // test responses and timeouts. Note that all of these tests are marked with the skip
-// decorator, since they take some time to run and this slows down local developement. To
+// decorator, since they take some time to run and this slows down local development. To
 // run them manually, pass the --no-skip flag when running tests. These tests are
 // run automatically as part of CI builds.
 
@@ -1649,14 +1647,14 @@ public:
 
 /**
  * Derived testing version of DNS_Mgr so that the Process() method can be exposed
- * publically. If new unit tests are added, this class should be used over using
+ * publicly. If new unit tests are added, this class should be used over using
  * DNS_Mgr directly.
  */
 class TestDNS_Mgr final : public DNS_Mgr
 	{
 public:
 	explicit TestDNS_Mgr(DNS_MgrMode mode) : DNS_Mgr(mode) { }
-	void Process();
+	void Process() override;
 	};
 
 void TestDNS_Mgr::Process()
