@@ -1,9 +1,10 @@
 // See the file "COPYING" in the main distribution directory for copyright.
 
-// We use deprecated APIs for MD5, SHA1 and SHA256. The reason is that, as of OpenSSL 3.0, there is
-// no API anymore that lets you store the internal state of hashing functions. For more information,
-// see https://github.com/zeek/zeek/issues/1379 and https://github.com/openssl/openssl/issues/14222
-// Since I don't feel like getting warnings every time we compile this file - let's silence them.
+// When using OpenSSL 3, we use deprecated APIs for MD5, SHA1 and SHA256. The reason is that, as of
+// OpenSSL 3.0, there is no API anymore that lets you store the internal state of hashing functions.
+// For more information, see https://github.com/zeek/zeek/issues/1379 and
+// https://github.com/openssl/openssl/issues/14222 Since I don't feel like getting warnings every
+// time we compile this file - let's silence them.
 
 #define OPENSSL_SUPPRESS_DEPRECATED
 
@@ -33,7 +34,7 @@ inline bool get_vector_idx(const V& v, unsigned int i, D* dst)
 	if ( i >= v.size() )
 		return false;
 
-	auto x = caf::get_if<S>(&v[i]);
+	auto x = broker::get_if<S>(&v[i]);
 	if ( ! x )
 		return false;
 
@@ -49,8 +50,6 @@ OpaqueMgr* OpaqueMgr::mgr()
 
 OpaqueVal::OpaqueVal(OpaqueTypePtr t) : Val(std::move(t)) { }
 
-OpaqueVal::~OpaqueVal() { }
-
 const std::string& OpaqueMgr::TypeID(const OpaqueVal* v) const
 	{
 	auto x = _types.find(v->OpaqueName());
@@ -81,12 +80,12 @@ broker::expected<broker::data> OpaqueVal::Serialize() const
 
 OpaqueValPtr OpaqueVal::Unserialize(const broker::data& data)
 	{
-	auto v = caf::get_if<broker::vector>(&data);
+	auto v = broker::get_if<broker::vector>(&data);
 
 	if ( ! (v && v->size() == 2) )
 		return nullptr;
 
-	auto type = caf::get_if<std::string>(&(*v)[0]);
+	auto type = broker::get_if<std::string>(&(*v)[0]);
 	if ( ! type )
 		return nullptr;
 
@@ -118,17 +117,17 @@ broker::expected<broker::data> OpaqueVal::SerializeType(const TypePtr& t)
 
 TypePtr OpaqueVal::UnserializeType(const broker::data& data)
 	{
-	auto v = caf::get_if<broker::vector>(&data);
+	auto v = broker::get_if<broker::vector>(&data);
 	if ( ! (v && v->size() == 2) )
 		return nullptr;
 
-	auto by_name = caf::get_if<bool>(&(*v)[0]);
+	auto by_name = broker::get_if<bool>(&(*v)[0]);
 	if ( ! by_name )
 		return nullptr;
 
 	if ( *by_name )
 		{
-		auto name = caf::get_if<std::string>(&(*v)[1]);
+		auto name = broker::get_if<std::string>(&(*v)[1]);
 		if ( ! name )
 			return nullptr;
 
@@ -142,7 +141,7 @@ TypePtr OpaqueVal::UnserializeType(const broker::data& data)
 		return id->GetType();
 		}
 
-	auto tag = caf::get_if<uint64_t>(&(*v)[1]);
+	auto tag = broker::get_if<uint64_t>(&(*v)[1]);
 	if ( ! tag )
 		return nullptr;
 
@@ -159,6 +158,16 @@ ValPtr OpaqueVal::DoClone(CloneState* state)
 	return state->NewClone(this, std::move(rval));
 	}
 
+void OpaqueVal::ValDescribe(ODesc* d) const
+	{
+	d->Add(util::fmt("<opaque of %s>", OpaqueName()));
+	}
+
+void OpaqueVal::ValDescribeReST(ODesc* d) const
+	{
+	d->Add(util::fmt("<opaque of %s>", OpaqueName()));
+	}
+
 bool HashVal::IsValid() const
 	{
 	return valid;
@@ -215,9 +224,18 @@ HashVal::HashVal(OpaqueTypePtr t) : OpaqueVal(std::move(t))
 	valid = false;
 	}
 
-MD5Val::MD5Val() : HashVal(md5_type) { }
+MD5Val::MD5Val() : HashVal(md5_type)
+	{
+	memset(&ctx, 0, sizeof(ctx));
+	}
 
-MD5Val::~MD5Val() { }
+MD5Val::~MD5Val()
+	{
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	if ( IsValid() )
+		EVP_MD_CTX_free(ctx);
+#endif
+	}
 
 void HashVal::digest_one(EVP_MD_CTX* h, const Val* v)
 	{
@@ -248,7 +266,11 @@ ValPtr MD5Val::DoClone(CloneState* state)
 		if ( ! out->Init() )
 			return nullptr;
 
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+		EVP_MD_CTX_copy_ex(out->ctx, ctx);
+#else
 		out->ctx = ctx;
+#endif
 		}
 
 	return state->NewClone(this, std::move(out));
@@ -257,7 +279,11 @@ ValPtr MD5Val::DoClone(CloneState* state)
 bool MD5Val::DoInit()
 	{
 	assert(! IsValid());
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	ctx = detail::hash_init(detail::Hash_MD5);
+#else
 	MD5_Init(&ctx);
+#endif
 	return true;
 	}
 
@@ -266,7 +292,11 @@ bool MD5Val::DoFeed(const void* data, size_t size)
 	if ( ! IsValid() )
 		return false;
 
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	detail::hash_update(ctx, data, size);
+#else
 	MD5_Update(&ctx, data, size);
+#endif
 	return true;
 	}
 
@@ -276,7 +306,11 @@ StringValPtr MD5Val::DoGet()
 		return val_mgr->EmptyString();
 
 	u_char digest[MD5_DIGEST_LENGTH];
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	detail::hash_final(ctx, digest);
+#else
 	MD5_Final(digest, &ctx);
+#endif
 	return make_intrusive<StringVal>(detail::md5_digest_print(digest));
 	}
 
@@ -287,7 +321,12 @@ broker::expected<broker::data> MD5Val::DoSerialize() const
 	if ( ! IsValid() )
 		return {broker::vector{false}};
 
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	MD5_CTX* md = (MD5_CTX*)EVP_MD_CTX_md_data(ctx);
+	auto data = std::string(reinterpret_cast<const char*>(md), sizeof(MD5_CTX));
+#else
 	auto data = std::string(reinterpret_cast<const char*>(&ctx), sizeof(ctx));
+#endif
 
 	broker::vector d = {true, data};
 	return {std::move(d)};
@@ -295,11 +334,11 @@ broker::expected<broker::data> MD5Val::DoSerialize() const
 
 bool MD5Val::DoUnserialize(const broker::data& data)
 	{
-	auto d = caf::get_if<broker::vector>(&data);
+	auto d = broker::get_if<broker::vector>(&data);
 	if ( ! d )
 		return false;
 
-	auto valid = caf::get_if<bool>(&(*d)[0]);
+	auto valid = broker::get_if<bool>(&(*d)[0]);
 	if ( ! valid )
 		return false;
 
@@ -312,21 +351,39 @@ bool MD5Val::DoUnserialize(const broker::data& data)
 	if ( (*d).size() != 2 )
 		return false;
 
-	auto s = caf::get_if<std::string>(&(*d)[1]);
+	auto s = broker::get_if<std::string>(&(*d)[1]);
 	if ( ! s )
 		return false;
 
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	if ( sizeof(MD5_CTX) != s->size() )
+#else
 	if ( sizeof(ctx) != s->size() )
+#endif
 		return false;
 
 	Init();
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	MD5_CTX* md = (MD5_CTX*)EVP_MD_CTX_md_data(ctx);
+	memcpy(md, s->data(), s->size());
+#else
 	memcpy(&ctx, s->data(), s->size());
+#endif
 	return true;
 	}
 
-SHA1Val::SHA1Val() : HashVal(sha1_type) { }
+SHA1Val::SHA1Val() : HashVal(sha1_type)
+	{
+	memset(&ctx, 0, sizeof(ctx));
+	}
 
-SHA1Val::~SHA1Val() { }
+SHA1Val::~SHA1Val()
+	{
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	if ( IsValid() )
+		EVP_MD_CTX_free(ctx);
+#endif
+	}
 
 ValPtr SHA1Val::DoClone(CloneState* state)
 	{
@@ -337,7 +394,11 @@ ValPtr SHA1Val::DoClone(CloneState* state)
 		if ( ! out->Init() )
 			return nullptr;
 
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+		EVP_MD_CTX_copy_ex(out->ctx, ctx);
+#else
 		out->ctx = ctx;
+#endif
 		}
 
 	return state->NewClone(this, std::move(out));
@@ -346,7 +407,11 @@ ValPtr SHA1Val::DoClone(CloneState* state)
 bool SHA1Val::DoInit()
 	{
 	assert(! IsValid());
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	ctx = detail::hash_init(detail::Hash_SHA1);
+#else
 	SHA1_Init(&ctx);
+#endif
 	return true;
 	}
 
@@ -355,7 +420,11 @@ bool SHA1Val::DoFeed(const void* data, size_t size)
 	if ( ! IsValid() )
 		return false;
 
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	detail::hash_update(ctx, data, size);
+#else
 	SHA1_Update(&ctx, data, size);
+#endif
 	return true;
 	}
 
@@ -365,7 +434,11 @@ StringValPtr SHA1Val::DoGet()
 		return val_mgr->EmptyString();
 
 	u_char digest[SHA_DIGEST_LENGTH];
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	detail::hash_final(ctx, digest);
+#else
 	SHA1_Final(digest, &ctx);
+#endif
 	return make_intrusive<StringVal>(detail::sha1_digest_print(digest));
 	}
 
@@ -376,7 +449,12 @@ broker::expected<broker::data> SHA1Val::DoSerialize() const
 	if ( ! IsValid() )
 		return {broker::vector{false}};
 
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	SHA_CTX* md = (SHA_CTX*)EVP_MD_CTX_md_data(ctx);
+	auto data = std::string(reinterpret_cast<const char*>(md), sizeof(SHA_CTX));
+#else
 	auto data = std::string(reinterpret_cast<const char*>(&ctx), sizeof(ctx));
+#endif
 
 	broker::vector d = {true, data};
 
@@ -385,11 +463,11 @@ broker::expected<broker::data> SHA1Val::DoSerialize() const
 
 bool SHA1Val::DoUnserialize(const broker::data& data)
 	{
-	auto d = caf::get_if<broker::vector>(&data);
+	auto d = broker::get_if<broker::vector>(&data);
 	if ( ! d )
 		return false;
 
-	auto valid = caf::get_if<bool>(&(*d)[0]);
+	auto valid = broker::get_if<bool>(&(*d)[0]);
 	if ( ! valid )
 		return false;
 
@@ -402,21 +480,39 @@ bool SHA1Val::DoUnserialize(const broker::data& data)
 	if ( (*d).size() != 2 )
 		return false;
 
-	auto s = caf::get_if<std::string>(&(*d)[1]);
+	auto s = broker::get_if<std::string>(&(*d)[1]);
 	if ( ! s )
 		return false;
 
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	if ( sizeof(SHA_CTX) != s->size() )
+#else
 	if ( sizeof(ctx) != s->size() )
+#endif
 		return false;
 
 	Init();
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	SHA_CTX* md = (SHA_CTX*)EVP_MD_CTX_md_data(ctx);
+	memcpy(md, s->data(), s->size());
+#else
 	memcpy(&ctx, s->data(), s->size());
+#endif
 	return true;
 	}
 
-SHA256Val::SHA256Val() : HashVal(sha256_type) { }
+SHA256Val::SHA256Val() : HashVal(sha256_type)
+	{
+	memset(&ctx, 0, sizeof(ctx));
+	}
 
-SHA256Val::~SHA256Val() { }
+SHA256Val::~SHA256Val()
+	{
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	if ( IsValid() )
+		EVP_MD_CTX_free(ctx);
+#endif
+	}
 
 ValPtr SHA256Val::DoClone(CloneState* state)
 	{
@@ -427,7 +523,11 @@ ValPtr SHA256Val::DoClone(CloneState* state)
 		if ( ! out->Init() )
 			return nullptr;
 
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+		EVP_MD_CTX_copy_ex(out->ctx, ctx);
+#else
 		out->ctx = ctx;
+#endif
 		}
 
 	return state->NewClone(this, std::move(out));
@@ -436,7 +536,11 @@ ValPtr SHA256Val::DoClone(CloneState* state)
 bool SHA256Val::DoInit()
 	{
 	assert(! IsValid());
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	ctx = detail::hash_init(detail::Hash_SHA256);
+#else
 	SHA256_Init(&ctx);
+#endif
 	return true;
 	}
 
@@ -445,7 +549,11 @@ bool SHA256Val::DoFeed(const void* data, size_t size)
 	if ( ! IsValid() )
 		return false;
 
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	detail::hash_update(ctx, data, size);
+#else
 	SHA256_Update(&ctx, data, size);
+#endif
 	return true;
 	}
 
@@ -455,7 +563,11 @@ StringValPtr SHA256Val::DoGet()
 		return val_mgr->EmptyString();
 
 	u_char digest[SHA256_DIGEST_LENGTH];
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	detail::hash_final(ctx, digest);
+#else
 	SHA256_Final(digest, &ctx);
+#endif
 	return make_intrusive<StringVal>(detail::sha256_digest_print(digest));
 	}
 
@@ -466,7 +578,12 @@ broker::expected<broker::data> SHA256Val::DoSerialize() const
 	if ( ! IsValid() )
 		return {broker::vector{false}};
 
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	SHA256_CTX* md = (SHA256_CTX*)EVP_MD_CTX_md_data(ctx);
+	auto data = std::string(reinterpret_cast<const char*>(md), sizeof(SHA256_CTX));
+#else
 	auto data = std::string(reinterpret_cast<const char*>(&ctx), sizeof(ctx));
+#endif
 
 	broker::vector d = {true, data};
 
@@ -475,11 +592,11 @@ broker::expected<broker::data> SHA256Val::DoSerialize() const
 
 bool SHA256Val::DoUnserialize(const broker::data& data)
 	{
-	auto d = caf::get_if<broker::vector>(&data);
+	auto d = broker::get_if<broker::vector>(&data);
 	if ( ! d )
 		return false;
 
-	auto valid = caf::get_if<bool>(&(*d)[0]);
+	auto valid = broker::get_if<bool>(&(*d)[0]);
 	if ( ! valid )
 		return false;
 
@@ -492,15 +609,24 @@ bool SHA256Val::DoUnserialize(const broker::data& data)
 	if ( (*d).size() != 2 )
 		return false;
 
-	auto s = caf::get_if<std::string>(&(*d)[1]);
+	auto s = broker::get_if<std::string>(&(*d)[1]);
 	if ( ! s )
 		return false;
 
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	if ( sizeof(SHA256_CTX) != s->size() )
+#else
 	if ( sizeof(ctx) != s->size() )
+#endif
 		return false;
 
 	Init();
+#if ( OPENSSL_VERSION_NUMBER < 0x30000000L ) || defined(LIBRESSL_VERSION_NUMBER)
+	SHA256_CTX* md = (SHA256_CTX*)EVP_MD_CTX_md_data(ctx);
+	memcpy(md, s->data(), s->size());
+#else
 	memcpy(&ctx, s->data(), s->size());
+#endif
 	return true;
 	}
 
@@ -546,7 +672,7 @@ broker::expected<broker::data> EntropyVal::DoSerialize() const
 
 bool EntropyVal::DoUnserialize(const broker::data& data)
 	{
-	auto d = caf::get_if<broker::vector>(&data);
+	auto d = broker::get_if<broker::vector>(&data);
 	if ( ! d )
 		return false;
 
@@ -611,6 +737,7 @@ ValPtr BloomFilterVal::DoClone(CloneState* state)
 	if ( bloom_filter )
 		{
 		auto bf = make_intrusive<BloomFilterVal>(bloom_filter->Clone());
+		assert(type);
 		bf->Typify(type);
 		return state->NewClone(this, std::move(bf));
 		}
@@ -638,6 +765,12 @@ void BloomFilterVal::Add(const Val* val)
 	bloom_filter->Add(key.get());
 	}
 
+bool BloomFilterVal::Decrement(const Val* val)
+	{
+	auto key = hash->MakeHashKey(*val, true);
+	return bloom_filter->Decrement(key.get());
+	}
+
 size_t BloomFilterVal::Count(const Val* val) const
 	{
 	auto key = hash->MakeHashKey(*val, true);
@@ -669,6 +802,8 @@ BloomFilterValPtr BloomFilterVal::Merge(const BloomFilterVal* x, const BloomFilt
 		return nullptr;
 		}
 
+	auto final_type = x->Type() ? x->Type() : y->Type();
+
 	if ( typeid(*x->bloom_filter) != typeid(*y->bloom_filter) )
 		{
 		reporter->Error("cannot merge different Bloom filter types");
@@ -686,7 +821,7 @@ BloomFilterValPtr BloomFilterVal::Merge(const BloomFilterVal* x, const BloomFilt
 
 	auto merged = make_intrusive<BloomFilterVal>(copy);
 
-	if ( x->Type() && ! merged->Typify(x->Type()) )
+	if ( final_type && ! merged->Typify(final_type) )
 		{
 		reporter->Error("failed to set type on merged Bloom filter");
 		return nullptr;
@@ -695,6 +830,42 @@ BloomFilterValPtr BloomFilterVal::Merge(const BloomFilterVal* x, const BloomFilt
 	return merged;
 	}
 
+BloomFilterValPtr BloomFilterVal::Intersect(const BloomFilterVal* x, const BloomFilterVal* y)
+	{
+	if ( x->Type() && // any one 0 is ok here
+	     y->Type() && ! same_type(x->Type(), y->Type()) )
+		{
+		reporter->Error("cannot merge Bloom filters with different types");
+		return nullptr;
+		}
+
+	if ( typeid(*x->bloom_filter) != typeid(*y->bloom_filter) )
+		{
+		reporter->Error("cannot intersect different Bloom filter types");
+		return nullptr;
+		}
+
+	auto intersected_bf = x->bloom_filter->Intersect(y->bloom_filter);
+
+	if ( ! intersected_bf )
+		{
+		reporter->Error("failed to intersect Bloom filter");
+		return nullptr;
+		}
+
+	auto final_type = x->Type() ? x->Type() : y->Type();
+
+	auto intersected = make_intrusive<BloomFilterVal>(intersected_bf);
+
+	if ( final_type && ! intersected->Typify(final_type) )
+		{
+		reporter->Error("Failed to set type on intersected bloom filter");
+		return nullptr;
+		}
+
+	return intersected;
+	}
+
 BloomFilterVal::~BloomFilterVal()
 	{
 	delete hash;
@@ -728,12 +899,12 @@ broker::expected<broker::data> BloomFilterVal::DoSerialize() const
 
 bool BloomFilterVal::DoUnserialize(const broker::data& data)
 	{
-	auto v = caf::get_if<broker::vector>(&data);
+	auto v = broker::get_if<broker::vector>(&data);
 
 	if ( ! (v && v->size() == 2) )
 		return false;
 
-	auto no_type = caf::get_if<broker::none>(&(*v)[0]);
+	auto no_type = broker::get_if<broker::none>(&(*v)[0]);
 	if ( ! no_type )
 		{
 		auto t = UnserializeType((*v)[0]);
@@ -822,12 +993,12 @@ broker::expected<broker::data> CardinalityVal::DoSerialize() const
 
 bool CardinalityVal::DoUnserialize(const broker::data& data)
 	{
-	auto v = caf::get_if<broker::vector>(&data);
+	auto v = broker::get_if<broker::vector>(&data);
 
 	if ( ! (v && v->size() == 2) )
 		return false;
 
-	auto no_type = caf::get_if<broker::none>(&(*v)[0]);
+	auto no_type = broker::get_if<broker::none>(&(*v)[0]);
 	if ( ! no_type )
 		{
 		auto t = UnserializeType((*v)[0]);
@@ -879,7 +1050,7 @@ broker::expected<broker::data> ParaglobVal::DoSerialize() const
 
 bool ParaglobVal::DoUnserialize(const broker::data& data)
 	{
-	auto d = caf::get_if<broker::vector>(&data);
+	auto d = broker::get_if<broker::vector>(&data);
 	if ( ! d )
 		return false;
 
