@@ -14,7 +14,7 @@ class BreakNextScriptValidation : public TraversalCallback
 public:
 	BreakNextScriptValidation(bool _report) : report(_report) { }
 
-	TraversalCode PreStmt(const Stmt* stmt)
+	TraversalCode PreStmt(const Stmt* stmt) override
 		{
 		if ( ! StmtIsRelevant(stmt) )
 			return TC_CONTINUE;
@@ -23,17 +23,15 @@ public:
 
 		if ( stmt->Tag() == STMT_BREAK && ! BreakStmtIsValid() )
 			Report(stmt, "break statement used outside of for, while or "
-			             "switch statement and not within a hook. "
-			             "With v6.1 this will become an error.");
+			             "switch statement and not within a hook.");
 
 		if ( stmt->Tag() == STMT_NEXT && ! NextStmtIsValid() )
-			Report(stmt, "next statement used outside of for or while statement. "
-			             "With v6.1 this will become an error.");
+			Report(stmt, "next statement used outside of for or while statement.");
 
 		return TC_CONTINUE;
 		}
 
-	TraversalCode PostStmt(const Stmt* stmt)
+	TraversalCode PostStmt(const Stmt* stmt) override
 		{
 		if ( ! StmtIsRelevant(stmt) )
 			return TC_CONTINUE;
@@ -45,7 +43,7 @@ public:
 		return TC_CONTINUE;
 		}
 
-	TraversalCode PreFunction(const zeek::Func* func)
+	TraversalCode PreFunction(const zeek::Func* func) override
 		{
 		if ( func->Flavor() == zeek::FUNC_FLAVOR_HOOK )
 			++hook_depth;
@@ -55,7 +53,7 @@ public:
 		return TC_CONTINUE;
 		}
 
-	TraversalCode PostFunction(const zeek::Func* func)
+	TraversalCode PostFunction(const zeek::Func* func) override
 		{
 		if ( func->Flavor() == zeek::FUNC_FLAVOR_HOOK )
 			--hook_depth;
@@ -65,6 +63,8 @@ public:
 		return TC_CONTINUE;
 		}
 
+	void SetHookDepth(int hd) { hook_depth = hd; }
+
 	bool IsValid() const { return valid_script; }
 
 private:
@@ -86,11 +86,7 @@ private:
 	void Report(const Stmt* stmt, const char* msg)
 		{
 		if ( report )
-			{
-			zeek::reporter->PushLocation(stmt->GetLocationInfo());
-			zeek::reporter->Warning("%s", msg);
-			zeek::reporter->PopLocation();
-			}
+			Error(stmt, msg);
 
 		valid_script = false;
 		}
@@ -107,10 +103,15 @@ void script_validation()
 	traverse_all(&bn_cb);
 	}
 
-bool script_is_valid(const Stmt* stmt)
+bool script_is_valid(const Stmt* stmt, bool is_in_hook)
 	{
 	BreakNextScriptValidation bn_cb(false);
+
+	if ( is_in_hook )
+		bn_cb.SetHookDepth(1);
+
 	stmt->Traverse(&bn_cb);
+
 	return bn_cb.IsValid();
 	}
 
