@@ -22,6 +22,7 @@ namespace zeek
 class Val;
 union ZVal;
 class EnumVal;
+class RecordVal;
 class TableVal;
 using ValPtr = IntrusivePtr<Val>;
 using EnumValPtr = IntrusivePtr<EnumVal>;
@@ -35,6 +36,19 @@ class ListExpr;
 class Attributes;
 using ListExprPtr = IntrusivePtr<ListExpr>;
 
+// The following tracks how to initialize a given record field.
+class FieldInit
+	{
+public:
+	virtual ~FieldInit() { }
+
+	// Return the initialization value of the field.
+	virtual ZVal Generate() const = 0;
+
+	// Can initialization of the field be deferred?
+	virtual bool IsDeferrable() const { return true; }
+	};
+
 	} // namespace detail
 
 // Zeek types.
@@ -500,10 +514,36 @@ public:
 	/**
 	 * A single lambda "capture" (outer variable used in a lambda's body).
 	 */
-	struct Capture
+	class Capture
 		{
-		detail::IDPtr id;
-		bool deep_copy;
+	public:
+		Capture(detail::IDPtr _id, bool _deep_copy);
+
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+		Capture(const Capture&) = default;
+		Capture(Capture&&) = default;
+		Capture& operator=(const Capture&) = default;
+		Capture& operator=(Capture&&) = default;
+		~Capture() = default;
+
+		auto& Id() const { return id; }
+		bool IsDeepCopy() const { return deep_copy; }
+		bool IsManaged() const { return is_managed; }
+
+		// For script optimization:
+		void SetID(detail::IDPtr new_id) { id = std::move(new_id); }
+#pragma GCC diagnostic pop
+
+		[[deprecated(
+			"Remove in v7.1.  Use non-default constructor and associated accessors.")]] detail::
+			IDPtr id;
+		[[deprecated(
+			"Remove in v7.1.  Use non-default constructor and associated accessors.")]] bool
+			deep_copy;
+		[[deprecated(
+			"Remove in v7.1.  Use non-default constructor and associated accessors.")]] bool
+			is_managed;
 		};
 
 	using CaptureList = std::vector<Capture>;
@@ -599,11 +639,6 @@ public:
 
 using type_decl_list = PList<TypeDecl>;
 
-// The following tracks how to initialize a given field.  We don't define
-// it here because it requires pulling in a bunch of low-level headers that
-// would be nice to avoid.
-class FieldInit;
-
 class RecordType final : public Type
 	{
 public:
@@ -682,15 +717,8 @@ public:
 
 	void AddFieldsDirectly(const type_decl_list& types, bool add_log_attr = false);
 
-	/**
-	 *
-	 * Populates a new instance of the record with its initial values.
-	 * @param r  The record's underlying value vector.
-	 */
-	void Create(std::vector<std::optional<ZVal>>& r) const;
-
 	void DescribeReST(ODesc* d, bool roles_only = false) const override;
-	void DescribeFields(ODesc* d) const;
+	void DescribeFields(ODesc* d, bool func_args = false) const;
 	void DescribeFieldsReST(ODesc* d, bool func_args) const;
 
 	bool IsFieldDeprecated(int field) const
@@ -709,16 +737,44 @@ public:
 
 	detail::TraversalCode Traverse(detail::TraversalCallback* cb) const override;
 
-protected:
+	// Can initialization of record values of this type be deferred?
+	//
+	// When record types contain non-const &default expressions or recursively
+	// contain any nested records that themselves are not deferrable,
+	// initialization can not be deferred, otherwise possible.
+	bool IsDeferrable() const;
+
+	// Whether values of this record type are equivalent upon initial
+	// creation, or might differ (e.g. due to function calls or changes
+	// to global state) - used for script optimization.
+	bool IdempotentCreation() const { return creation_inits.empty(); }
+
+	static void InitPostScript();
+
+private:
 	RecordType() { types = nullptr; }
 
 	void AddField(unsigned int field, const TypeDecl* td);
 
 	void DoDescribe(ODesc* d) const override;
 
-	// Maps each field to how to initialize it.  Uses pointers due to
-	// keeping the FieldInit definition private to Type.cc (see above).
-	std::vector<FieldInit*> field_inits;
+	// Field initializations that can be deferred to first access,
+	// beneficial for fields that are separately initialized prior
+	// to first access.  Nil pointers mean "skip initializing the field".
+	std::vector<std::unique_ptr<detail::FieldInit>> deferred_inits;
+
+	// Field initializations that need to be done upon record creation,
+	// rather than deferred.  These are expressions whose value might
+	// change if computed later.
+	//
+	// Such initializations are uncommon, so we represent them using
+	// <fieldoffset, init> pairs.
+	std::vector<std::pair<int, std::unique_ptr<detail::FieldInit>>> creation_inits;
+
+	class CreationInitsOptimizer;
+	friend zeek::RecordVal;
+	const auto& DeferredInits() const { return deferred_inits; }
+	const auto& CreationInits() const { return creation_inits; }
 
 	// If we were willing to bound the size of records, then we could
 	// use std::bitset here instead.
@@ -928,12 +984,6 @@ extern TypeTag max_type(TypeTag t1, TypeTag t2);
 // an error message) if the types are incompatible.
 TypePtr merge_types(const TypePtr& t1, const TypePtr& t2);
 
-// Given a list of expressions, returns a (ref'd) type reflecting
-// a merged type consistent across all of them, or nil if this
-// cannot be done.
-[[deprecated("Remove in v6.1. Use maximal_type() if possible. See GH-2604.")]] TypePtr
-merge_type_list(detail::ListExpr* elements);
-
 // Given a list of expressions, returns the maximal type consistent across
 // all of them, or nil if this cannot be done.  "Maximal" incorporates
 // notions of arithmetic coercion, but otherwise requires type-equivalence.
@@ -1008,7 +1058,7 @@ inline bool IsString(TypeTag t)
 	return (t == TYPE_STRING);
 	}
 
-// True if the given type is a container aggregate.
+// True if the given type is an aggregate.
 inline bool IsAggr(TypeTag tag)
 	{
 	return tag == TYPE_VECTOR || tag == TYPE_TABLE || tag == TYPE_RECORD;
@@ -1022,6 +1072,12 @@ inline bool IsAggr(const TypePtr& t)
 	return IsAggr(t->Tag());
 	}
 
+// True if the given type is a container.
+inline bool IsContainer(TypeTag tag)
+	{
+	return tag == TYPE_VECTOR || tag == TYPE_TABLE;
+	}
+
 // True if the given type tag corresponds to the error type.
 inline bool IsErrorType(TypeTag t)
 	{
