@@ -38,14 +38,18 @@ extern void add_global(const IDPtr& id, TypePtr t, InitClass c, ExprPtr init,
 extern StmtPtr add_local(IDPtr id, TypePtr t, InitClass c, ExprPtr init,
                          std::unique_ptr<std::vector<AttrPtr>> attr, DeclType dt);
 
-extern ExprPtr add_and_assign_local(IDPtr id, ExprPtr init, ValPtr val = nullptr);
+extern ExprPtr add_and_assign_local(IDPtr id, ExprPtr init, ValPtr val);
 
 extern void add_type(ID* id, TypePtr t, std::unique_ptr<std::vector<AttrPtr>> attr);
 
+extern void add_module(const char* module_name);
+
+const std::set<std::string>& module_names();
+
 extern void begin_func(IDPtr id, const char* module_name, FunctionFlavor flavor, bool is_redef,
                        FuncTypePtr t, std::unique_ptr<std::vector<AttrPtr>> attrs = nullptr);
 
-extern void end_func(StmtPtr body, bool free_of_conditionals);
+extern void end_func(StmtPtr body, const char* module_name, bool free_of_conditionals);
 
 // Gather all IDs referenced inside a body that aren't part of a given scope.
 extern IDPList gather_outer_ids(ScopePtr scope, StmtPtr body);
