@@ -434,17 +434,39 @@ public:
 	 *
 	 * @param tag The type of analyzer to prevent.
 	 */
-	void PreventChildren(zeek::Tag tag);
+	void PreventChildren(const zeek::Tag& tag);
+
+	/**
+	 * Returns true if the given analyzer type is prevented from
+	 * being added as a child.
+	 *
+	 * @param tag The type of analyzer to prevent.
+	 *
+	 * @return true if the analyzer type is prevented, else false.
+	 */
+	bool IsPreventedChildAnalyzer(const zeek::Tag& tag) const;
 
 	/**
 	 * Returns true if analyzer has a direct child of a given type.
 	 *
 	 * @param tag The type of analyzer to check for.
 	 */
-	bool HasChildAnalyzer(zeek::Tag tag);
+	bool HasChildAnalyzer(const zeek::Tag& tag) const;
 
 	/**
-	 * Recursively searches all (direct or indirect) childs of the
+	 * Returns a pointer to a direct child analyzer of the given type.
+	 *
+	 * Note that the returned pointer is owned by the analyzer and may
+	 * be deleted without notification. Do not hold on to it.
+	 *
+	 * @param tag The type of the analyzer to check for.
+	 *
+	 * @return The analyzer, or null if not found.
+	 */
+	Analyzer* GetChildAnalyzer(const zeek::Tag& tag) const;
+
+	/**
+	 * Recursively searches all (direct or indirect) children of the
 	 * analyzer for an analyzer with a specific ID.
 	 *
 	 * @param id The analyzer id to search. This is the ID that GetID()
@@ -455,7 +477,7 @@ public:
 	virtual Analyzer* FindChild(ID id);
 
 	/**
-	 * Recursively searches all (direct or indirect) childs of the
+	 * Recursively searches all (direct or indirect) children of the
 	 * analyzer for an analyzer of a given type.
 	 *
 	 * @param tag The analyzer type to search.
@@ -466,7 +488,7 @@ public:
 	virtual Analyzer* FindChild(zeek::Tag tag);
 
 	/**
-	 * Recursively searches all (direct or indirect) childs of the
+	 * Recursively searches all (direct or indirect) children of the
 	 * analyzer for an analyzer of a given type.
 	 *
 	 * @param name The name of the analyzer type to search (e.g.,
@@ -715,17 +737,10 @@ private:
 	// Internal helper to raise analyzer_confirmation events
 	void EnqueueAnalyzerConfirmationInfo(const zeek::Tag& arg_tag);
 
-	// Remove in v6.1 - internal helper to raise analyzer_confirmation
-	void EnqueueAnalyzerConfirmation(const zeek::Tag& arg_tag);
-
 	// Internal helper to raise analyzer_violation_info
 	void EnqueueAnalyzerViolationInfo(const char* reason, const char* data, int len,
 	                                  const zeek::Tag& arg_tag);
 
-	// Remove in v6.1 - internal helper to raise analyzer_violation
-	void EnqueueAnalyzerViolation(const char* reason, const char* data, int len,
-	                              const zeek::Tag& arg_tag);
-
 	zeek::Tag tag;
 	ID id;
 
