@@ -76,13 +76,6 @@ void HTTP_Entity::EndOfData()
 		encoding = IDENTITY;
 		}
 
-	zeek::detail::Rule::PatternType rule = http_message->IsOrig()
-	                                           ? zeek::detail::Rule::HTTP_REQUEST_BODY
-	                                           : zeek::detail::Rule::HTTP_REPLY_BODY;
-
-	http_message->MyHTTP_Analyzer()->Conn()->Match(rule, reinterpret_cast<const u_char*>(""), 0,
-	                                               http_message->IsOrig(), false, true, false);
-
 	if ( body_length )
 		http_message->MyHTTP_Analyzer()->ForwardEndOfData(http_message->IsOrig());
 
@@ -1300,7 +1293,7 @@ int HTTP_Analyzer::HTTP_RequestLine(const char* line, const char* end_of_line)
 
 	// If we determined HTTP/0.9 (no HTTP/ in the request line), assert that
 	// minimally we have an URI and a 3 character method (HTTP 0.9 only
-	// supports GET). If that doesn't hold, probably not HTTP or very stange.
+	// supports GET). If that doesn't hold, probably not HTTP or very strange.
 	if ( request_version == HTTP_VersionNumber{0, 9} )
 		{
 		bool maybe_get_method = (end_of_method - line) >= 3;
