@@ -130,7 +130,7 @@ public:
 	string file_id;
 
 	AnalysisStream();
-	~AnalysisStream() override;
+	~AnalysisStream() override = default;
 	};
 
 Manager::TableStream::TableStream()
@@ -177,8 +177,6 @@ Manager::TableStream::~TableStream()
 
 Manager::AnalysisStream::AnalysisStream() : Manager::Stream::Stream(ANALYSIS_STREAM), file_id() { }
 
-Manager::AnalysisStream::~AnalysisStream() { }
-
 Manager::Manager() : plugin::ComponentManager<input::Component>("Input", "Reader")
 	{
 	end_of_data = event_registry->Register("Input::end_of_data");
@@ -653,7 +651,7 @@ bool Manager::CreateTableStream(RecordVal* fval)
 	if ( ! CheckErrorEventTypes(stream_name, error_event, true) )
 		return false;
 
-	vector<Field*> fieldsV; // vector, because we don't know the length beforehands
+	vector<Field*> fieldsV; // vector, because we don't know the length beforehand
 
 	bool status = (! UnrollRecordType(&fieldsV, idx, "", false));
 
@@ -1106,9 +1104,10 @@ Val* Manager::ValueToIndexVal(const Stream* i, int num_fields, const RecordType*
 
 					const char* warning = "Skipping input with missing non-optional value";
 					if ( source && file_pos != -1 )
-						Warning(i, "%s:%d: %s", source, file_pos, warning);
+						Warning(i, "%s:%d: %s (index field %s)", source, file_pos, warning,
+						        type->FieldName(j));
 					else
-						Warning(i, "%s", warning);
+						Warning(i, "%s (index field %s)", warning, type->FieldName(j));
 
 					have_error = true;
 					}
@@ -1963,9 +1962,10 @@ RecordVal* Manager::ValueToRecordVal(const Stream* stream, const Value* const* v
 
 			const char* warning = "Skipping input with missing non-optional value";
 			if ( source && file_pos != -1 )
-				Warning(stream, "%s:%d: %s", source, file_pos, warning);
+				Warning(stream, "%s:%d: %s (record field %s)", source, file_pos, warning,
+				        request_type->FieldName(i));
 			else
-				Warning(stream, "%s", warning);
+				Warning(stream, "%s (record field %s)", warning, request_type->FieldName(i));
 
 			have_error = true;
 			}
