@@ -2,20 +2,25 @@
 
 #pragma once
 
-extern "C" {
+#include <unistd.h>
+
+extern "C"
+	{
 #include <pcap.h>
-}
+	}
 
 #include "zeek/iosource/PktDumper.h"
 
-namespace zeek::iosource::pcap {
+namespace zeek::iosource::pcap
+	{
 
-class PcapDumper : public PktDumper {
+class PcapDumper : public PktDumper
+	{
 public:
 	PcapDumper(const std::string& path, bool append);
-	~PcapDumper() override;
+	~PcapDumper() override = default;
 
-	static PktDumper* Instantiate(const std::string& path, bool appen);
+	static PktDumper* Instantiate(const std::string& path, bool append);
 
 protected:
 	// PktDumper interface.
@@ -29,6 +34,6 @@ private:
 	bool append;
 	pcap_dumper_t* dumper;
 	pcap_t* pd;
-};
+	};
 
-} // namespace zeek::iosource::pcap
+	} // namespace zeek::iosource::pcap
