@@ -80,6 +80,7 @@ string CPPCompile::GenExpr(const Expr* e, GenType gt, bool top_level)
 		case EXPR_TIMES:
 			return GenBinary(e, gt, "*", "mul");
 		case EXPR_DIVIDE:
+		case EXPR_MASK: // later code will split into addr masking
 			return GenBinary(e, gt, "/", "div");
 		case EXPR_MOD:
 			return GenBinary(e, gt, "%", "mod");
@@ -114,7 +115,7 @@ string CPPCompile::GenExpr(const Expr* e, GenType gt, bool top_level)
 		case EXPR_COND:
 			return GenCondExpr(e, gt);
 		case EXPR_CALL:
-			return GenCallExpr(e->AsCallExpr(), gt);
+			return GenCallExpr(e->AsCallExpr(), gt, top_level);
 		case EXPR_LIST:
 			return GenListExpr(e, gt, false);
 		case EXPR_IN:
@@ -291,7 +292,7 @@ string CPPCompile::GenCondExpr(const Expr* e, GenType gt)
 	return string("(") + gen1 + ") ? (" + gen2 + ") : (" + gen3 + ")";
 	}
 
-string CPPCompile::GenCallExpr(const CallExpr* c, GenType gt)
+string CPPCompile::GenCallExpr(const CallExpr* c, GenType gt, bool top_level)
 	{
 	const auto& t = c->GetType();
 	auto f = c->Func();
@@ -347,7 +348,18 @@ string CPPCompile::GenCallExpr(const CallExpr* c, GenType gt)
 		// Indirect call.
 		gen = string("(") + gen + ")->AsFunc()";
 
-	string invoke_func = is_async ? "when_invoke__CPP" : "invoke__CPP";
+	string invoke_func;
+
+	if ( is_async )
+		invoke_func = "when_invoke__CPP";
+	else if ( top_level || t->Tag() == TYPE_VOID )
+		{
+		ASSERT(top_level);
+		invoke_func = "invoke_void__CPP";
+		}
+	else
+		invoke_func = "invoke__CPP";
+
 	auto args_list = string(", {") + GenExpr(args_l, GEN_VAL_PTR) + "}";
 	auto invoker = invoke_func + "(" + gen + args_list + ", f__CPP";
 
@@ -356,6 +368,10 @@ string CPPCompile::GenCallExpr(const CallExpr* c, GenType gt)
 
 	invoker += ")";
 
+	if ( top_level )
+		// No need to use accessor.
+		return invoker;
+
 	if ( IsNativeType(t) && gt != GEN_VAL_PTR )
 		return invoker + NativeAccessor(t);
 
@@ -526,7 +542,9 @@ string CPPCompile::GenAddToExpr(const Expr* e, GenType gt, bool top_level)
 
 	if ( t->Tag() == TYPE_VECTOR )
 		{
-		if ( same_type(lhs->GetType(), rhs->GetType()) )
+		auto& rt = rhs->GetType();
+
+		if ( IsVector(rt->Tag()) && same_type(lhs->GetType(), rt) )
 			add_to_func = "vector_vec_append__CPP";
 		else
 			add_to_func = "vector_append__CPP";
@@ -1043,7 +1061,7 @@ string CPPCompile::GenBinaryAddr(const Expr* e, GenType gt, const char* op)
 	{
 	auto v1 = GenExpr(e->GetOp1(), GEN_DONT_CARE) + "->AsAddr()";
 
-	if ( e->Tag() == EXPR_DIVIDE )
+	if ( e->Tag() == EXPR_MASK )
 		{
 		auto gen = string("addr_mask__CPP(") + v1 + ", " + GenExpr(e->GetOp2(), GEN_NATIVE) + ")";
 
@@ -1286,7 +1304,7 @@ string CPPCompile::GenLambdaClone(const LambdaExpr* l, bool all_deep)
 		if ( captures && ! IsNativeType(id_t) )
 			{
 			for ( const auto& c : *captures )
-				if ( id == c.id && (c.deep_copy || all_deep) )
+				if ( id == c.Id() && (c.IsDeepCopy() || all_deep) )
 					arg = string("cast_intrusive<") + TypeName(id_t) + ">(" + arg + "->Clone())";
 			}
 
@@ -1337,7 +1355,7 @@ string CPPCompile::GenField(const ExprPtr& rec, int field)
 		ASSERT(pt != processed_types.end());
 		auto rt_offset = pt->second->Offset();
 		string field_name = rt->FieldName(field);
-		field_decls.emplace_back(pair(rt_offset, rt->FieldDecl(field)));
+		field_decls.emplace_back(rt_offset, rt->FieldDecl(field));
 
 		if ( rfm != record_field_mappings.end() )
 			// We're already tracking this record.
@@ -1377,7 +1395,7 @@ string CPPCompile::GenEnum(const TypePtr& t, const ValPtr& ev)
 		mapping_slot = num_ev_mappings++;
 
 		string enum_name = et->Lookup(v);
-		enum_names.emplace_back(pair(TypeOffset(t), std::move(enum_name)));
+		enum_names.emplace_back(TypeOffset(t), std::move(enum_name));
 
 		if ( evm != enum_val_mappings.end() )
 			{
