@@ -60,6 +60,7 @@ class ZAMCompiler
 public:
 	ZAMCompiler(ScriptFunc* f, std::shared_ptr<ProfileFunc> pf, ScopePtr scope, StmtPtr body,
 	            std::shared_ptr<UseDefs> ud, std::shared_ptr<Reducer> rd);
+	~ZAMCompiler();
 
 	StmtPtr CompileBody();
 
@@ -92,6 +93,7 @@ private:
 	void Init();
 	void InitGlobals();
 	void InitArgs();
+	void InitCaptures();
 	void InitLocals();
 	void TrackMemoryManagement();
 
@@ -123,8 +125,6 @@ private:
 	const ZAMStmt CompileStmt(const StmtPtr& body) { return CompileStmt(body.get()); }
 	const ZAMStmt CompileStmt(const Stmt* body);
 
-	void SetCurrStmt(const Stmt* stmt) { curr_stmt = stmt; }
-
 	const ZAMStmt CompilePrint(const PrintStmt* ps);
 	const ZAMStmt CompileExpr(const ExprStmt* es);
 	const ZAMStmt CompileIf(const IfStmt* is);
@@ -220,10 +220,15 @@ private:
 	const ZAMStmt CompileInExpr(const NameExpr* n1, const ListExpr* l, const NameExpr* n2,
 	                            const ConstExpr* c);
 
-	const ZAMStmt CompileIndex(const NameExpr* n1, const NameExpr* n2, const ListExpr* l);
-	const ZAMStmt CompileIndex(const NameExpr* n1, const ConstExpr* c, const ListExpr* l);
+	const ZAMStmt CompileIndex(const NameExpr* n1, const NameExpr* n2, const ListExpr* l,
+	                           bool in_when);
+	const ZAMStmt CompileIndex(const NameExpr* n1, const ConstExpr* c, const ListExpr* l,
+	                           bool in_when);
 	const ZAMStmt CompileIndex(const NameExpr* n1, int n2_slot, const TypePtr& n2_type,
-	                           const ListExpr* l);
+	                           const ListExpr* l, bool in_when);
+
+	const ZAMStmt BuildLambda(const NameExpr* n, LambdaExpr* le);
+	const ZAMStmt BuildLambda(int n_slot, LambdaExpr* le);
 
 	// Second argument is which instruction slot holds the branch target.
 	const ZAMStmt GenCond(const Expr* e, int& branch_v);
@@ -334,7 +339,10 @@ private:
 	// Returns how many values were added.
 	int InternalAddVal(ZInstAux* zi, int i, Expr* e);
 
-	const ZAMStmt AddInst(const ZInstI& inst);
+	// Adds the given instruction to the ZAM program.  The second
+	// argument, if true, suppresses generation of any pending
+	// global/capture store for this instruction.
+	const ZAMStmt AddInst(const ZInstI& inst, bool suppress_non_local = false);
 
 	// Returns the statement just before the given one.
 	ZAMStmt PrevStmt(const ZAMStmt s);
@@ -348,8 +356,15 @@ private:
 
 	bool IsUnused(const IDPtr& id, const Stmt* where) const;
 
+	bool IsCapture(const IDPtr& id) const { return IsCapture(id.get()); }
+	bool IsCapture(const ID* id) const;
+
+	int CaptureOffset(const IDPtr& id) const { return IsCapture(id.get()); }
+	int CaptureOffset(const ID* id) const;
+
 	void LoadParam(const ID* id);
 	const ZAMStmt LoadGlobal(const ID* id);
+	const ZAMStmt LoadCapture(const ID* id);
 
 	int AddToFrame(const ID*);
 
@@ -443,14 +458,6 @@ private:
 	const ZInstI* BeginningOfLoop(const ZInstI* inst, int depth) const;
 	const ZInstI* EndOfLoop(const ZInstI* inst, int depth) const;
 
-	// True if any statement other than a frame sync assigns to the
-	// given slot.
-	bool VarIsAssigned(int slot) const;
-
-	// True if the given statement assigns to the given slot, and
-	// it's not a frame sync.
-	bool VarIsAssigned(int slot, const ZInstI* i) const;
-
 	// True if any statement other than a frame sync uses the given slot.
 	bool VarIsUsed(int slot) const;
 
@@ -597,8 +604,10 @@ private:
 
 	// Used for communication between Frame1Slot and a subsequent
 	// AddInst.  If >= 0, then upon adding the next instruction,
-	// it should be followed by Store-Global for the given slot.
+	// it should be followed by Store-Global or Store-Capture for
+	// the given slot.
 	int pending_global_store = -1;
+	int pending_capture_store = -1;
 	};
 
 // Invokes after compiling all of the function bodies.
