@@ -128,7 +128,7 @@ int ZAMCompiler::InternalAddVal(ZInstAux* zi, int i, Expr* e)
 	return 1;
 	}
 
-const ZAMStmt ZAMCompiler::AddInst(const ZInstI& inst)
+const ZAMStmt ZAMCompiler::AddInst(const ZInstI& inst, bool suppress_non_local)
 	{
 	ZInstI* i;
 
@@ -146,17 +146,46 @@ const ZAMStmt ZAMCompiler::AddInst(const ZInstI& inst)
 
 	top_main_inst = insts1.size() - 1;
 
-	if ( pending_global_store < 0 )
+	if ( suppress_non_local )
 		return ZAMStmt(top_main_inst);
 
-	auto global_slot = pending_global_store;
-	pending_global_store = -1;
+	// Ensure we haven't confused ourselves about any pending stores.
+	ASSERT(pending_global_store == -1 || pending_capture_store == -1);
 
-	auto store_inst = ZInstI(OP_STORE_GLOBAL_V, global_slot);
-	store_inst.op_type = OP_V_I1;
-	store_inst.t = globalsI[global_slot].id->GetType();
+	if ( pending_global_store >= 0 )
+		{
+		auto gs = pending_global_store;
+		pending_global_store = -1;
 
-	return AddInst(store_inst);
+		auto store_inst = ZInstI(OP_STORE_GLOBAL_V, gs);
+		store_inst.op_type = OP_V_I1;
+		store_inst.t = globalsI[gs].id->GetType();
+
+		return AddInst(store_inst);
+		}
+
+	if ( pending_capture_store >= 0 )
+		{
+		auto cs = pending_capture_store;
+		pending_capture_store = -1;
+
+		auto& cv = *func->GetType()->AsFuncType()->GetCaptures();
+		auto& c_id = cv[cs].Id();
+
+		ZOp op;
+
+		if ( ZVal::IsManagedType(c_id->GetType()) )
+			op = OP_STORE_MANAGED_CAPTURE_VV;
+		else
+			op = OP_STORE_CAPTURE_VV;
+
+		auto store_inst = ZInstI(op, RawSlot(c_id.get()), cs);
+		store_inst.op_type = OP_VV_I2;
+
+		return AddInst(store_inst);
+		}
+
+	return ZAMStmt(top_main_inst);
 	}
 
 const Stmt* ZAMCompiler::LastStmt(const Stmt* s) const
@@ -164,7 +193,8 @@ const Stmt* ZAMCompiler::LastStmt(const Stmt* s) const
 	if ( s->Tag() == STMT_LIST )
 		{
 		auto sl = s->AsStmtList()->Stmts();
-		return sl[sl.length() - 1];
+		ASSERT(! sl.empty());
+		return sl.back().get();
 		}
 
 	else
