@@ -13,7 +13,7 @@ namespace zeek::detail
 
 const ZAMStmt ZAMCompiler::CompileStmt(const Stmt* s)
 	{
-	SetCurrStmt(s);
+	curr_stmt = const_cast<Stmt*>(s)->ThisPtr();
 
 	switch ( s->Tag() )
 		{
@@ -60,6 +60,9 @@ const ZAMStmt ZAMCompiler::CompileStmt(const Stmt* s)
 		case STMT_INIT:
 			return CompileInit(static_cast<const InitStmt*>(s));
 
+		case STMT_WHEN:
+			return CompileWhen(static_cast<const WhenStmt*>(s));
+
 		case STMT_NULL:
 			return EmptyStmt();
 
@@ -834,7 +837,7 @@ const ZAMStmt ZAMCompiler::LoopOverTable(const ForStmt* f, const NameExpr* val)
 		aux->value_var_type = value_var->GetType();
 
 	auto iter_slot = table_iters.size();
-	table_iters.emplace_back(TableIterInfo());
+	table_iters.emplace_back();
 
 	auto z = ZInstI(OP_INIT_TABLE_LOOP_VV, FrameSlot(val), iter_slot);
 	z.op_type = OP_VV_I2;
@@ -1047,14 +1050,7 @@ const ZAMStmt ZAMCompiler::CompileCatchReturn(const CatchReturnStmt* cr)
 
 	ResolveCatchReturns(GoToTargetBeyond(block_end));
 
-	// If control flow runs off the end of the block, then we need
-	// to consider sync'ing globals at that point.
-	auto block_last = LastStmt(block.get());
-
-	if ( block_last->Tag() == STMT_RETURN )
-		return block_end;
-
-	return top_main_inst;
+	return block_end;
 	}
 
 const ZAMStmt ZAMCompiler::CompileStmts(const StmtList* ws)
@@ -1100,6 +1096,60 @@ const ZAMStmt ZAMCompiler::CompileInit(const InitStmt* is)
 	return last;
 	}
 
+const ZAMStmt ZAMCompiler::CompileWhen(const WhenStmt* ws)
+	{
+	auto wi = ws->Info();
+	auto timeout = wi->TimeoutExpr();
+
+	auto lambda = NewSlot(true);
+	(void)BuildLambda(lambda, wi->Lambda().get());
+
+	std::vector<IDPtr> local_aggr_slots;
+	for ( auto& l : wi->WhenExprLocals() )
+		if ( IsAggr(l->GetType()->Tag()) )
+			local_aggr_slots.push_back(l);
+
+	int n = local_aggr_slots.size();
+	auto aux = new ZInstAux(n);
+	aux->wi = wi;
+
+	for ( auto i = 0; i < n; ++i )
+		{
+		auto la = local_aggr_slots[i];
+		aux->Add(i, FrameSlot(la), la->GetType());
+		}
+
+	ZInstI z;
+
+	if ( timeout )
+		{
+		if ( timeout->Tag() == EXPR_NAME )
+			{
+			auto ns = FrameSlot(timeout->AsNameExpr());
+			z = ZInstI(OP_WHEN_TIMEOUT_VV, lambda, ns);
+			}
+		else
+			{
+			ASSERT(timeout->Tag() == EXPR_CONST);
+			z = ZInstI(OP_WHEN_TIMEOUT_VC, lambda, timeout->AsConstExpr());
+			}
+		}
+
+	else
+		z = ZInstI(OP_WHEN_V, lambda);
+
+	z.aux = aux;
+
+	if ( ws->IsReturn() )
+		{
+		(void)AddInst(z);
+		z = ZInstI(OP_RETURN_C);
+		z.c = ZVal();
+		}
+
+	return AddInst(z);
+	}
+
 const ZAMStmt ZAMCompiler::InitRecord(IDPtr id, RecordType* rt)
 	{
 	auto z = ZInstI(OP_INIT_RECORD_V, FrameSlot(id));
