@@ -554,14 +554,8 @@ SetupResult setup(int argc, char** argv, Options* zopts)
 	auto stem = Supervisor::CreateStem(options.supervisor_mode);
 
 	if ( Supervisor::ThisNode() )
-		{
-		// If we get here, we're a supervised node that just returned
-		// from CreateStem() after being forked from the stem.
 		Supervisor::ThisNode()->Init(&options);
 
-		event_mgr.InitPostFork();
-		}
-
 	script_coverage_mgr.ReadStats();
 
 	auto dns_type = options.dns_mode;
@@ -826,13 +820,16 @@ SetupResult setup(int argc, char** argv, Options* zopts)
 		// when we actually end up reading interactively from stdin.
 		set_signal_mask(false);
 		run_state::is_parsing = true;
-		yyparse();
+		int yyparse_result = yyparse();
 		run_state::is_parsing = false;
 		set_signal_mask(true);
 
 		RecordVal::DoneParsing();
 		TableVal::DoneParsing();
 
+		if ( yyparse_result != 0 || zeek::reporter->Errors() > 0 )
+			exit(1);
+
 		init_general_global_var();
 		init_net_var();
 		run_bif_initializers();
@@ -876,6 +873,8 @@ SetupResult setup(int argc, char** argv, Options* zopts)
 		if ( reporter->Errors() > 0 )
 			exit(1);
 
+		RecordType::InitPostScript();
+
 		telemetry_mgr->InitPostScript();
 		iosource_mgr->InitPostScript();
 		log_mgr->InitPostScript();
@@ -1121,6 +1120,8 @@ SetupResult setup(int argc, char** argv, Options* zopts)
 			g_frame_stack.pop_back();
 			}
 
+		clear_script_analysis();
+
 		if ( zeek_script_loaded )
 			{
 			// Queue events reporting loaded scripts.
