@@ -13,6 +13,7 @@
 #include "zeek/Reporter.h"
 #include "zeek/util.h"
 #include "zeek/zeekygen/Configuration.h"
+#include "zeek/zeekygen/SpicyModuleInfo.h"
 
 namespace zeek
 	{
@@ -77,7 +78,7 @@ public:
 
 	/**
 	 * Do initialization that needs to happen after scripts are parsed.
-	 * This is primarly dependency resolution/filtering.
+	 * This is primarily dependency resolution/filtering.
 	 */
 	void InitPostScript();
 
@@ -102,7 +103,7 @@ public:
 	void ScriptDependency(const std::string& path, const std::string& dep);
 
 	/**
-	 * Register a module usage (script may export identifiers in to the
+	 * Register a module usage (script may export identifiers into the
 	 * module namespace).
 	 * @param path Absolute path to a Zeek script.
 	 * @param module The module which script given by \a path is using.
@@ -141,13 +142,25 @@ public:
 	 * @param id The identifier being redef'd.
 	 * @param path Absolute path to a Zeek script doing the redef.
 	 * @param ic The initialization class that was used (e.g. =, +=, -=).
-	 * @param init_expr The intiialization expression that was used.
+	 * @param init_expr The initialization expression that was used.
 	 */
 	void Redef(const zeek::detail::ID* id, const std::string& path, zeek::detail::InitClass ic,
 	           zeek::detail::ExprPtr init_expr);
 	void Redef(const zeek::detail::ID* id, const std::string& path,
 	           zeek::detail::InitClass ic = zeek::detail::INIT_NONE);
 
+	/**
+	 * Register a Spicy EVT module.
+	 * @param info the module information
+	 */
+	void AddSpicyModule(std::unique_ptr<SpicyModuleInfo> info)
+		{
+		spicy_modules.map[info->Name()] = info.get();
+		all_info.push_back(info.release()); // switch to manual memory mgmt like all other infos
+		}
+
+	const auto& SpicyModules() const { return spicy_modules.map; }
+
 	/**
 	 * Register Zeekygen script summary content.
 	 * @param path Absolute path to a Zeek script.
@@ -199,7 +212,7 @@ public:
 	/**
 	 * @param name Name of a Zeek script package ("normalized" to be a path
 	 * relative to a component within ZEEKPATH).
-	 * @return a package info object assocated with \a name or a null pointer
+	 * @return a package info object associated with \a name or a null pointer
 	 * if it's not a known package name.
 	 */
 	PackageInfo* GetPackageInfo(const std::string& name) const { return packages.GetInfo(name); }
@@ -215,6 +228,9 @@ public:
 	template <class T>
 	bool IsUpToDate(const std::string& target_file, const std::vector<T*>& dependencies) const;
 
+	/** Returns true if Zeekygen is enabled. */
+	bool IsEnabled() const { return ! disabled; }
+
 private:
 	using comment_buffer_t = std::vector<std::string>;
 	using comment_buffer_map_t = std::map<std::string, comment_buffer_t>;
@@ -228,6 +244,7 @@ private:
 	InfoMap<PackageInfo> packages;
 	InfoMap<ScriptInfo> scripts;
 	InfoMap<IdentifierInfo> identifiers;
+	InfoMap<SpicyModuleInfo> spicy_modules;
 	std::vector<Info*> all_info;
 	IdentifierInfo* last_identifier_seen;
 	IdentifierInfo* incomplete_type;
